@@ -1,7 +1,7 @@
-# -*- coding: ISO-8859-1 -*-
+# -*- encoding: utf-8 -*-
 #
 #
-# Copyright (C) 2004-2005 Andr Wobst <wobsta@users.sourceforge.net>
+# Copyright (C) 2004-2012 André Wobst <wobsta@users.sourceforge.net>
 #
 # This file is part of PyX (http://pyx.sourceforge.net/).
 #
@@ -20,12 +20,13 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
 
+import importlib
 __allmodules__ = ["data", "key", "style", "axis"]
 for module in __allmodules__:
-    __import__(module, globals(), locals(), [])
+    importlib.import_module('.' + module, package='pyx.graph')
 
-import graph
-__allgraph__ = ["graphxy", "graphxyz"]
+from . import graph
+__allgraph__ = ["graphx", "graphxy", "graphxyz"]
 for importfromgraph in __allgraph__:
     locals()[importfromgraph] = getattr(graph, importfromgraph)
 
