@@ -3,7 +3,7 @@
 #
 # Copyright (C) 2002-2004 Jörg Lehmann <joergl@users.sourceforge.net>
 # Copyright (C) 2003-2004 Michael Schindler <m-schindler@users.sourceforge.net>
-# Copyright (C) 2002-2006 André Wobst <wobsta@users.sourceforge.net>
+# Copyright (C) 2002-2012 André Wobst <wobsta@users.sourceforge.net>
 #
 # This file is part of PyX (http://pyx.sourceforge.net/).
 #
@@ -21,10 +21,10 @@
 # along with PyX; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
-import math, re, ConfigParser, struct, warnings
+import math, re, configparser, struct
 from pyx import text
-from pyx.style import linestyle
-from pyx.graph import style
+from . import style
+builtinlist = list
 
 
 def splitatvalue(value, *splitpoints):
@@ -66,13 +66,13 @@ _mathglobals = {"neg": lambda x: -x,
 class _data:
     """graph data interface
 
-    Graph data consists in columns, where each column might be identified by a
+    Graph data consists of columns, where each column might be identified by a
     string or an integer. Each row in the resulting table refers to a data
     point.
 
     All methods except for the constructor should consider self and its
     attributes to be readonly, since the data instance might be shared between
-    several graphs simultaniously.
+    several graphs simultaneously.
 
     The instance variable columns is a dictionary mapping column names to the
     data of the column (i.e. to a list). Only static columns (known at
@@ -85,15 +85,19 @@ class _data:
     and stated in the columnnames dictionary.
 
     The instance variable title and defaultstyles contain the data title and
-    the default styles (a list of styles), respectively.
+    the default styles (a list of styles), respectively. If defaultstyles is None,
+    the data cannot be plotted without user provided styles.
     """
 
-    def dynamiccolumns(self, graph):
+    def dynamiccolumns(self, graph, axisnames):
         """create and return dynamic columns data
 
         Returns dynamic data matching the given axes (the axes range and other
         data might be used). The return value is a dictionary similar to the
-        columns instance variable.
+        columns instance variable. However, the static and dynamic data does
+        not need to be correlated in any way, i.e. the number of data points in
+        self.columns might differ from the number of data points represented by
+        the return value of the dynamiccolumns method.
         """
         return {}
 
@@ -107,13 +111,13 @@ class values(_data):
     defaultstyles = defaultsymbols
 
     def __init__(self, title="user provided values", **columns):
-        for i, values in enumerate(columns.values()):
+        for i, values in enumerate(list(columns.values())):
             if i and len(values) != l:
                 raise ValueError("different number of values")
             else:
                 l = len(values)
         self.columns = columns
-        self.columnnames = columns.keys()
+        self.columnnames = list(columns.keys())
         self.title = title
 
 
@@ -131,23 +135,18 @@ class points(_data):
                     raise ValueError("different number of columns per point")
                 for i, x in enumerate(point):
                     self.columndata[i].append(x)
-            for v in columns.values():
+            for v in list(columns.values()):
                 if abs(v) > l or (not addlinenumbers and abs(v) == l):
                     raise ValueError("column number bigger than number of columns")
             if addlinenumbers:
-                self.columndata = [range(1, len(points) + 1)] + self.columndata
-            self.columns = dict([(key, self.columndata[i]) for key, i in columns.items()])
+                self.columndata = [list(range(1, len(points) + 1))] + self.columndata
+            self.columns = dict([(key, self.columndata[i]) for key, i in list(columns.items())])
         else:
-            self.columns = dict([(key, []) for key, i in columns.items()])
-        self.columnnames = self.columns.keys()
+            self.columns = dict([(key, []) for key, i in list(columns.items())])
+        self.columnnames = list(self.columns.keys())
         self.title = title
 
 
-def list(*args, **kwargs):
-    warnings.warn("graph.data.list is deprecated. Use graph.data.points instead.")
-    return points(*args, **kwargs)
-
-
 class _notitle:
     pass
 
@@ -160,7 +159,7 @@ class data(_data):
                        replacedollar=1, columncallback="__column__", **columns):
         # build a nice title
         if title is _notitle:
-            items = columns.items()
+            items = list(columns.items())
             items.sort() # we want sorted items (otherwise they would be unpredictable scrambled)
             self.title = "%s: %s" % (text.escapestring(data.title or "unkown source"),
                                      ", ".join(["%s=%s" % (text.escapestring(key),
@@ -174,7 +173,7 @@ class data(_data):
 
         # analyse the **columns argument
         self.columns = {}
-        for columnname, value in columns.items():
+        for columnname, value in list(columns.items()):
             # search in the columns dictionary
             try:
                 self.columns[columnname] = self.orgdata.columns[value]
@@ -195,16 +194,16 @@ class data(_data):
                     context = context.copy()
                     context[columncallback] = self.columncallback
                     if self.orgdata.columns:
-                        key, columndata = self.orgdata.columns.items()[0]
+                        key, columndata = list(self.orgdata.columns.items())[0]
                         count = len(columndata)
                     elif self.orgdata.columndata:
                         count = len(self.orgdata.columndata[0])
                     else:
                         count = 0
                     newdata = []
-                    for i in xrange(count):
+                    for i in range(count):
                         self.columncallbackcount = i
-                        for key, values in self.orgdata.columns.items():
+                        for key, values in list(self.orgdata.columns.items()):
                             context[key] = values[i]
                         try:
                             newdata.append(eval(expression, _mathglobals, context))
@@ -214,11 +213,11 @@ class data(_data):
 
         if copy:
             # copy other, non-conflicting column names
-            for columnname, columndata in self.orgdata.columns.items():
-                if not self.columns.has_key(columnname):
+            for columnname, columndata in list(self.orgdata.columns.items()):
+                if columnname not in self.columns:
                     self.columns[columnname] = columndata
 
-        self.columnnames = self.columns.keys()
+        self.columnnames = list(self.columns.keys())
 
     def columncallback(self, value):
         try:
@@ -265,7 +264,7 @@ class file(data):
         else:
             if tofloat:
                 try:
-                    return map(float, line.split())
+                    return list(map(float, line.split()))
                 except (TypeError, ValueError):
                     result = []
                     for r in line.split():
@@ -312,7 +311,7 @@ class file(data):
             if skiptail >= every:
                 skip, x = divmod(skiptail, every)
                 del columndata[-skip:]
-            for i in xrange(len(columndata)):
+            for i in range(len(columndata)):
                 if len(columndata[i]) != maxcolumns:
                     columndata[i].extend([None]*(maxcolumns-len(columndata[i])))
             return points(columndata, title=title, addlinenumbers=0,
@@ -323,8 +322,9 @@ class file(data):
         except:
             # not a file-like object -> open it
             cachekey = self.getcachekey(filename, commentpattern, stringpattern, columnpattern, skiphead, skiptail, every)
-            if not filecache.has_key(cachekey):
-                filecache[cachekey] = readfile(open(filename), filename)
+            if cachekey not in filecache:
+                with open(filename) as f:
+                    filecache[cachekey] = readfile(f, filename)
             data.__init__(self, filecache[cachekey], **kwargs)
         else:
             data.__init__(self, readfile(filename, "user provided file-like object"), **kwargs)
@@ -345,15 +345,15 @@ class conffile(data):
           keyword arguments data and titles excluded"""
 
         def readfile(file, title):
-            config = ConfigParser.ConfigParser()
+            config = configparser.ConfigParser(strict=False)
             config.optionxform = str
-            config.readfp(file)
+            config.read_file(file)
             sections = config.sections()
             sections.sort()
             columndata = [None]*len(sections)
             maxcolumns = 1
             columns = {}
-            for i in xrange(len(sections)):
+            for i in range(len(sections)):
                 point = [sections[i]] + [None]*(maxcolumns-1)
                 for option in config.options(sections[i]):
                     value = config.get(sections[i], option)
@@ -380,7 +380,7 @@ class conffile(data):
             filename.readlines
         except:
             # not a file-like object -> open it
-            if not filecache.has_key(filename):
+            if filename not in filecache:
                 filecache[filename] = readfile(open(filename), filename)
             data.__init__(self, filecache[filename], **kwargs)
         else:
@@ -464,9 +464,9 @@ class cbdfile(data):
             # remove jumps at long +/- 180
             for sd, sb in zip(sds, sbs):
                 if sd.minlong < -150*3600 and sd.maxlong > 150*3600:
-                    for i, (lat, long) in enumerate(sb.points):
-                         if long < 0:
-                             sb.points[i] = lat, long + 360*3600
+                    for i, (lat, int) in enumerate(sb.points):
+                         if int < 0:
+                             sb.points[i] = lat, int + 360*3600
 
             columndata = []
             for sd, sb in zip(sds, sbs):
@@ -474,8 +474,8 @@ class cbdfile(data):
                     (maxrank is None or sd.rank <= maxrank)):
                     if columndata:
                         columndata.append((None, None))
-                    columndata.extend([(long/3600.0, lat/3600.0)
-                                       for lat, long in sb.points])
+                    columndata.extend([(int/3600.0, lat/3600.0)
+                                       for lat, int in sb.points])
 
             result = points(columndata, title=title)
             result.defaultstyles = self.defaultstyles
@@ -487,7 +487,7 @@ class cbdfile(data):
         except:
             # not a file-like object -> open it
             cachekey = self.getcachekey(filename, minrank, maxrank)
-            if not cbdfilecache.has_key(cachekey):
+            if cachekey not in cbdfilecache:
                 cbdfilecache[cachekey] = readfile(open(filename, "rb"), filename)
             data.__init__(self, cbdfilecache[cachekey], **kwargs)
         else:
@@ -517,16 +517,16 @@ class function(_data):
             expression = expression[m.end():]
         else:
             raise ValueError("y(x)=... or similar expected")
-        if context.has_key(self.xname):
+        if self.xname in context:
             raise ValueError("xname in context")
         self.expression = compile(expression.strip(), __file__, "eval")
         self.columns = {}
         self.columnnames = [self.xname, self.yname]
 
-    def dynamiccolumns(self, graph):
+    def dynamiccolumns(self, graph, axisnames):
         dynamiccolumns = {self.xname: [], self.yname: []}
 
-        xaxis = graph.axes[self.xname]
+        xaxis = graph.axes[axisnames.get(self.xname, self.xname)]
         from pyx.graph.axis import logarithmic
         logaxis = isinstance(xaxis.axis, logarithmic)
         if self.min is not None:
@@ -565,7 +565,7 @@ class paramfunction(_data):
     defaultstyles = defaultlines
 
     def __init__(self, varname, min, max, expression, title=_notitle, points=100, context={}):
-        if context.has_key(varname):
+        if varname in context:
             raise ValueError("varname in context")
         if title is _notitle:
             self.title = expression
@@ -584,10 +584,71 @@ class paramfunction(_data):
                 self.columns[key].append(value)
         if len(keys) != len(values):
             raise ValueError("unpack tuple of wrong size")
-        self.columnnames = self.columns.keys()
+        self.columnnames = list(self.columns.keys())
 
 
 class paramfunctionxy(paramfunction):
 
     def __init__(self, f, min, max, **kwargs):
         paramfunction.__init__(self, "t", min, max, "x, y = f(t)", context={"f": f}, **kwargs)
+
+
+class _nodefaultstyles:
+    pass
+
+
+class join(_data):
+    "creates a new data set by joining from a list of data, it does however *not* combine points, but fills data with None if necessary"
+
+    def merge_lists(self, lists):
+        "merges list items w/o duplications, resulting order is arbitraty"
+        result = set()
+        for l in lists:
+            result.update(set(l))
+        return builtinlist(result)
+
+    def merge_dicts(self, dicts):
+        """merge dicts containing lists as values (with equal number of items
+        per list in each dict), missing data is padded by None"""
+        keys = self.merge_lists([list(d.keys()) for d in dicts])
+        empties = []
+        for d in dicts:
+            if len(list(d.keys())) == len(keys):
+                empties.append(None) # won't be needed later on
+            else:
+                values = list(d.values())
+                if len(values):
+                    empties.append([None]*len(values[0]))
+                else:
+                    # has no data at all -> do not add anything
+                    empties.append([])
+        result = {}
+        for key in keys:
+            result[key] = []
+            for d, e in zip(dicts, empties):
+                result[key].extend(d.get(key, e))
+        return result
+
+    def __init__(self, data, title=_notitle, defaultstyles=_nodefaultstyles):
+        """takes a list of data, a title (if it should not be autoconstructed)
+        and a defaultstyles list if there is no common defaultstyles setting
+        for in the provided data"""
+        assert len(data)
+        self.data = data
+        self.columnnames = self.merge_lists([d.columnnames for d in data])
+        self.columns = self.merge_dicts([d.columns for d in data])
+        if title is _notitle:
+            self.title = " + ".join([d.title for d in data])
+        else:
+            self.title = title
+        if defaultstyles is _nodefaultstyles:
+            self.defaultstyles = data[0].defaultstyles
+            for d in data[1:]:
+                if d.defaultstyles is not self.defaultstyles:
+                    self.defaultstyles = None
+                    break
+        else:
+            self.defaultstyles = defaultstyles
+
+    def dynamiccolumns(self, graph, axisnames):
+        return self.merge_dicts([d.dynamiccolumns(graph, axisnames) for d in self.data])
