@@ -1,10 +1,9 @@
-#!/usr/bin/env python
-# -*- coding: ISO-8859-1 -*-
+# -*- encoding: utf-8 -*-
 #
 #
-# Copyright (C) 2002-2004 Jrg Lehmann <joergl@users.sourceforge.net>
+# Copyright (C) 2002-2004 Jörg Lehmann <joergl@users.sourceforge.net>
 # Copyright (C) 2003-2004 Michael Schindler <m-schindler@users.sourceforge.net>
-# Copyright (C) 2002-2004 Andr Wobst <wobsta@users.sourceforge.net>
+# Copyright (C) 2002-2011 André Wobst <wobsta@users.sourceforge.net>
 #
 # This file is part of PyX (http://pyx.sourceforge.net/).
 #
@@ -20,10 +19,10 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with PyX; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
 
-from pyx import box, canvas, text, unit
+from pyx import box, canvas, text, trafo, unit
 
 
 class key:
@@ -33,7 +32,8 @@ class key:
     def __init__(self, dist=0.2*unit.v_cm, pos="tr", hpos=None, vpos=None,
                  hinside=1, vinside=1, hdist=0.6*unit.v_cm, vdist=0.4*unit.v_cm,
                  symbolwidth=0.5*unit.v_cm, symbolheight=0.25*unit.v_cm, symbolspace=0.2*unit.v_cm,
-                 textattrs=[], border=0.3*unit.v_cm, keyattrs=None):
+                 textattrs=[], columns=1, columndist=0.5*unit.v_cm,
+                 border=0.3*unit.v_cm, keyattrs=None):
         self.dist = dist
         self.hinside = hinside
         self.vinside = vinside
@@ -43,6 +43,8 @@ class key:
         self.symbolheight = symbolheight
         self.symbolspace = symbolspace
         self.textattrs = textattrs
+        self.columns = columns
+        self.columndist = columndist
         self.border = border
         self.keyattrs = keyattrs
         if pos is not None:
@@ -69,9 +71,8 @@ class key:
             self.hpos = hpos
             self.vpos = vpos
 
-    def paint(self, plotitems):
-        "creates the layout of the key"
-        plotitems = [plotitem for plotitem in plotitems if plotitem.gettitle() is not None]
+    def paintcolumn(self, plotitems):
+        "creates the layout of a key column"
         c = canvas.canvas()
         self.dist_pt = unit.topt(self.dist)
         self.hdist_pt = unit.topt(self.hdist)
@@ -81,7 +82,7 @@ class key:
         self.symbolspace_pt = unit.topt(self.symbolspace)
         titleboxes = []
         for plotitem in plotitems:
-            titlebox = c.texrunner.text_pt(0, 0, plotitem.gettitle(), self.defaulttextattrs + self.textattrs)
+            titlebox = c.texrunner.text_pt(0, 0, plotitem.title, self.defaulttextattrs + self.textattrs)
             titlebox.plotitem = plotitem
             titleboxes.append(titlebox)
         dy_pt = box.tile_pt(titleboxes, self.dist_pt, 0, -1)
@@ -92,6 +93,19 @@ class key:
             y_pt -= dy_pt
         for titlebox in titleboxes:
             c.insert(titlebox)
+        return c
+
+    def paint(self, plotitems):
+        "creates the layout of the key"
+        columndist_pt = unit.topt(self.columndist)
+        c = canvas.canvas()
+        itemspercolumn = (len(plotitems) + self.columns - 1) // self.columns
+        x_pt = 0
+        while plotitems:
+            subc = self.paintcolumn(plotitems[:itemspercolumn])
+            c.insert(subc, [trafo.translate_pt(x_pt, 0)])
+            x_pt += unit.topt(subc.bbox().width()) + columndist_pt
+            del plotitems[:itemspercolumn]
         if self.keyattrs is not None:
             newc = canvas.canvas()
             newc.draw(c.bbox().enlarged(self.border).path(), self.keyattrs)
